/* FILE: labelvalue.h                 -*-Mode: c++-*-
 *
 * Concrete label-value class, derived from OXS extension class.
 * A wrapper around an STL map object, using the OXS initialization
 * paradigm.
 *
 */

#ifndef _OXS_LABELVALUE
#define _OXS_LABELVALUE

#include <string>
#include <map>

#include "ext.h"
/* End includes */

class Oxs_LabelValue:public Oxs_Ext {
private:
  // Uses parent "init_strings" map.
public:
  virtual const char* ClassName() const; // ClassName() is
  /// automatically generated by the OXS_EXT_REGISTER macro.
  Oxs_LabelValue(const char* name,     // Child instance id
		 Oxs_Director* newdtr, // App director
		 const char* argstr);  // MIF input block parameters

  BOOL Init() { return 1; }

  virtual ~Oxs_LabelValue() {}

  // Read access
  void GetLabels(vector<string>& labels) const;

  BOOL HasLabel(const string& label) const;  // Returns 1 if label is
  /// a valid key into init_strings, 0 otherwise.

  string GetValue(const string& label) const; // Throws an exception if
  /// "label" is not a key in init_strings.

  // Write access
  void SetLabel(const string& label,const string& value);
  /// Unconditionally sets/adds the pair label,value to init_strings.

  BOOL InitializeLabel(const string& label,const string& value);
  /// If 'label' is not in init_strings, then adds it with associated
  /// value 'value', and returns 1.  If 'label' is already in
  /// init_strings, then the value is unchanged and 0 is returned.

};

#endif // _OXS_LABELVALUE
