# FILE: procs.tcl
#
# Misc. Tcl support code for the mmsolve application.
#

# Proc to read a file.  This is a wrapper for use by safe
# interpreters, and also provides image conversion facility.
# The second argument should be one of "binary", "auto" or
# "image".  The first two govern the translation mode of the
# opened file.  Selecting "image" causes the input to be
# converted to mimic the body of a PPM P3 text file---the first
# 3 whitespace separated values are width, height, and maxvalue.
# After this is an array or pixels, r g b.  The difference
# compared to a straight P3 file is that the leading "P3" is
# removed, and there are no comments.  The output can be treated
# as a list and used directly.
#
# The return value is a string representing the read data.
#
proc Oxs_ReadFile { file {translation auto} } {
    if {![string match image $translation]} {
	# Standard file read
	set chanid [open $file r]
	if {[string match binary $translation]} {
	    fconfigure $chanid -translation binary
	}
	fconfigure $chanid -buffering full -buffersize 50000
	set filestr [read $chanid]
	close $chanid
	return $filestr
    } else {
	# Read and convert to simple text representation
	set chanid [open $file r]
	fconfigure $chanid -translation binary \
		-buffering full -buffersize 250000
	set type_check [read $chanid 2]
	if {![string match P3 $type_check]} {
	    # Input file is not PPM P3 (ASCII) type.
	    # Send through converter.
	    close $chanid
	    set var {}
	    if {[catch {
		Oc_Application CommandLine any2ppm -noinfo -o - $file
	    } cmdLine]} {
		return -code error "Can't find any2ppm program to\
			convert	image file $file to PPM P3 format."
	    }
	    set cmdLine [linsert $cmdLine 0 |]
	    # Ignore output set the stderr.  Without this redirection,
	    # any output to stderr causes Tcl to raise an error on
	    # closing the channel.  In particular, some debugging
	    # libraries print out version information at program start
	    # on stderr.
	    lappend cmdLine "2>" \
                    [[Oc_Config RunPlatform] GetValue path_device_null]
	    set chanid [open $cmdLine r]
	    fconfigure $chanid -translation binary \
		    -buffering full -buffersize 250000
	    set type_check [read $chanid 2]
	    if {![string match P3 $type_check]} {
		return -code error "Conversion of image file $file to\
			PPM P3 format failed."
	    }
	}
	set image [read $chanid] ;# Note that leading P3 already stripped
	close $chanid
    
	# Remove comments, if any
	regsub -all -- "#\[^\n]*\n" $image "\n" image
    
	return $image
    }
}

